##
## ocean.nas
##
##  Nasal code for managing ocean related random objects.
##
##  This file is licensed under the GPL license.
##    Anders Gidenstam  2007

UPDATE_INTERVAL = 10;
NMtoM = 1852;

##
# Singleton class that manages icebergs. It's started with icebergs.init()
# and then works autonomously.
#
var icebergs = {
    
    init : func {
        ## Iceberg areas: (center, radius(nm))
        me.areas = [
                    [geo.Coord.new().set_lonlat(0,  90), 1380], # Arctica
                    [geo.Coord.new().set_lonlat(0, -90), 2100], # Antarctica
                    [geo.Coord.new().set_lonlat(-40, 70), 900]  # Greenland
                   ];
        me.enabled =
            props.globals.getNode("/environment/ocean/icebergs/enabled", 1);
        me.present =
            props.globals.getNode("/environment/ocean/icebergs/present", 1);
        me.loopid = 0;
        _setlistener("/environment/ocean/icebergs/enabled",
                     func { icebergs.update(); });
	me.reset();
	print("Ocean.icebergs ... initialized");
    },
    reset : func {
        me.loopid += 1;
        me._loop_(me.loopid);
    },
    iceberg_area : func(pos) {
        foreach(a; me.areas) {
#            print("(" ~ pos.lon() ~ "," ~ pos.lat() ~ "), (" ~
#                  a[0].lon() ~ "," ~ a[0].lat() ~ "):" ~ a[1]);
#            print(" dist = " ~ pos.distance_to(a[0]));
            if (pos.distance_to(a[0]) < NMtoM * a[1]) {
                return 1==1;
            }
        }
        return 0==1;
    },
    update : func {
        var pos = geo.aircraft_position();
        me.present.setValue(me.iceberg_area(pos) and me.enabled.getValue());
#        print("Ocean.icebergs ... refreshed pos " ~
#              pos.lon() ~ "," ~ pos.lat() ~ " -> " ~ me.present.getValue());
    },
    _loop_ : func(id) {
        id == me.loopid or return;
        me.update();
        settimer(func { me._loop_(id); }, UPDATE_INTERVAL);
    }
};

_setlistener("/sim/signals/fdm-initialized", func {
    icebergs.init();
});
